/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 */

package org.objectweb.dream.dreamannotation.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.List;

import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.generator.Generator;
import org.objectweb.fractal.fraclet.annotation.processor.util.ADLFileFactory;

import spoon.reflect.declaration.CtClass;

/**
 * An ADL generator for components defined with the <code>@DreamComponent
 * </code> annotation. <br />
 * <br />
 * The <strong>only</strong> difference with the fraclet version is that
 * {@link FractalComponent} is replaced here by {@link DreamComponent}
 * 
 * @author Pierre Garcia <Pierre.Garcia@inrialpes.fr>
 */
public class DreamComponentADLGenerator
{

  private PrintWriter pw = null;

  public DreamComponentADLGenerator(CtClass<?> processedClass,
      DreamComponent dcAnnotation, List<Generator> generators)
  {

    // -----------------------------------------------------
    // create .fractal file
    // -----------------------------------------------------
    File outFile = ADLFileFactory.createADLFile(processedClass.getFactory()
        .getEnvironment().getDefaultFileGenerator().getOutputDirectory(),
        processedClass);
    try
    {
      pw = new PrintWriter(outFile);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }

    for (Generator g : generators)
      g.prettyprint(pw, processedClass);

    // -----------------------------------------------------
    // Print Controller Desc
    // -----------------------------------------------------

    pw
        .println("  <controller desc=\"" + dcAnnotation.controllerDesc()
            + "\"/>");

    // -----------------------------------------------------
    // closing file
    // -----------------------------------------------------

    pw.println("</definition>");
    pw.close();

  }
}
